import logging
import sys

from flask import Flask
from flask import Response
from flask import request

from .pdf import reservation_order_a4
from .pdf import reservation_bon
from .pdf import retro_a4

# Wir arbeiten hier mit Handlern und filtern um ein einigermaßen gutes Logging für den Container zu bekommen.
# Die ist einfach: Für das Loglevel INFO gehen wir nach stdout, höheres nach stderr. Daher filtern wir beim Handler für Level INFO.
formatter = logging.Formatter("%(name)s:%(lineno)d: %(message)s")
std_err_handler = logging.StreamHandler(sys.stderr)
std_err_handler.setLevel(logging.WARNING)
std_err_handler.setFormatter(formatter)

std_out_handler = logging.StreamHandler(sys.stdout)
std_out_handler.addFilter(lambda x: x.levelno <= logging.INFO)
std_out_handler.setFormatter(formatter)

logger = logging.getLogger(__name__)

logger.setLevel(logging.INFO)
logger.addHandler(std_err_handler)
logger.addHandler(std_out_handler)

app = Flask(__name__)


@app.route("/", methods=["POST"])
def make_pdf():
    """
    Wählt eine geeignete Funktion anhand der print_group_name aus und erzeugt eine PDF-Antwort
    :return: Response
    """
    logger.debug(request.json)
    print_request = request.json[0]
    logger.info("Bearbeite Slip %s" % print_request.get("id"))
    print_group_name = print_request.get("print_group_name", "")
    if not print_group_name:
        logger.warning("print_group_name ist leer")
        logger.info(print_request)
    if print_group_name == "Phil II Vormerkungen":
        func = reservation_bon
    elif print_group_name == "retro":
        func = retro_a4
    # Die restlichen Zettel sehen alle gleich aus, daher diese "unsaubere" Lösung.
    else:
        func = reservation_order_a4
    logger.info(
        "Slip %s in Queue %s, nutze Funktion %s"
        % (print_request.get("id"), print_group_name, func.__name__)
    )
    # Wir geben nur die eigentlichen Druckdaten weiter. Die Metadaten wie z.B. den print_group_name brauchen wir nicht mehr.
    body = func(print_request["data"])
    # Wir müssen wieder an den Anfang des BytesIO, bevor wir die Antwort schicken.
    body.seek(0)
    return Response(body, content_type="application/pdf")
